# Manage user settings.

import json as _json
import os as _os
import pyperclip as _pclp
from pathlib import Path as _Path

# import subprocess as _subp


def setting_handler() -> dict:
    """Handle the user settings by creating and/or verifing a user setting file in the JSON format."""

    # The path starting from the user home directory up to the directory that host the .json setting file
    sp: _Path = _Path.home() / _Path(".Worddfn/conf/")

    # The absolute path of the .json setting file.
    sf: _Path = sp / "setting.json"

    if sf.exists():
        with open(sf, "r") as setting:
            usrConfig = _json.load(setting)

    else:
        _os.makedirs(sp)
        sf.touch()

        # Define the user setting to serialize to a json format.
        fileContent: dict = setting_prompt()
        print("Configuring client.....")

        with open(sf, "w") as setting:
            _json.dump(fileContent, setting)

        print(f"All Done.")
        _pclp.copy("")

        # Remove the use of subprocess calling gedit not multi-platform, every thing should be handle from the cli.
        # print(
        #     "Add an API KEY in the api_key section, also chose if to handle from cli or use pyperclip."
        # )
        # _subp.call(["gedit", sf])

        with open(sf, "r") as setting:
            usrConfig = _json.load(setting)

    return usrConfig


def setting_prompt() -> dict:
    """Prompt user to add the necessary configuration when the client is run for the first time."""
    apiKey: str = ""
    handler: str = ""
    fileContent: dict = {}

    print(f"Please insert (or copy to your clipboard) a Wordnik API key:", end=" ")
    if len(_pclp.paste()) > 0:
        apiKey = _pclp.paste()
        print(f"'{apiKey}'")
    else:
        apiKey = input(f"")

    handler = input(
        f"Enter a Handler [clipboard|default] or [cli] | Press Enter for default:"
    )

    while handler != "clipboard" and handler != "cli":
        if handler == "" or handler == "default":
            handler = "clipboard"
            break
        else:
            handler = input(
                f"Must enter one of two option [clipboard] or [cli] or press enter for clipboard:"
            )
            if handler == "" or handler == "default":
                handler = "clipboard"

    fileContent = {"api_key": apiKey, "handler": handler}
    return fileContent


if __name__ == "__main__":
    print(f"Executing from {__name__}")



# Help Create objects for different Dictionary API's.


class WordnikConfig:
    """Create objects for the wordnik api."""

    def __init__(self) -> None:
        self.url = ""
        self.headers = {}
        self.parameters = {}

    def show_api_obj(self) -> str:
        """Print the Object url, header and parameter. Simple put it display if the object is empty or not."""

        return f"Object contains:\nurl:{self.url}\nheaders:{self.headers}\nparameters:{self.parameters}"

    def headers_construct(self, accp: str, API_KEY: str) -> dict:
        """Build part of the HTTP request header.
        accp:        Accepts type: e.g. application/json
        API_KEY:     API KEY: uses an Wordnik API KEY.
        This method returns a dictionary"""

        return {
            "Accepts": accp,
            "api_key": API_KEY,
        }

    def parameters_construct(self, endPoint: str, *query: tuple[str]) -> dict:
        """Builds out the parameters for the HTTP request, returns a dictionary
        This method takes a wordnik api endpoint and optionaly can be pass a
        list of query varibles witch the api uses to get specific results."""

        # Create parameters use for calling the definitions endpoint.
        if endPoint == "definitions":
            limit: str = query[0]
            src: str = query[1]
            tag: str = query[2]

            parameters = {
                "limit": limit,
                "sourceDictionaries": src,
                "includeTags": tag,
            }

        # Create parameters use for calling the relatedWords endpoint.
        elif endPoint == "relatedWords":
            # Create parameters use for calling the relatedWords endpoint when exactly two query variable is pass.
            if len(query) == 2:
                word_relations: srt = query[0]
                limit_relations: srt = query[1]

                parameters = {
                    "relationshipTypes": word_relations,
                    "limitPerRelationshipType": limit_relations,
                }

            # Create parameters use for calling the relatedWords endpoint when exactly one query variable is pass.
            elif len(query) == 1:
                val = query[0]
                # Check to see if the querry have numbers in it.
                if val.isdecimal():
                    # print(type(val))  # test code
                    limit_relations: str = query[0]

                    parameters = {
                        "relationshipTypes": "verb-form",
                        "limitPerRelationshipType": limit_relations,
                    }

                # Check to see if the querry is compose of only string.
                elif val.isalpha():
                    # print(type(val))  # test code
                    word_relations: str = query[0]

                    parameters = {
                        "relationshipTypes": word_relations,
                        "limitPerRelationshipType": "5",
                    }

            else:  # Test code
                print("something wrong")  # Test code

        return parameters

    def url_construct(self, URL: str, word: str, endPoint: str) -> str:
        """Builds a full wordnik api URL to get results.
             URL:         the wordnik api base url.
             word:        a word to search in the dictionary for.
             endPoint:    the wordnik api endpoint.
        this nethod returns a string with the fully created wordnik api url."""

        url = f"{URL}{word}/{endPoint}"
        return url

    def api_construct(self, url: str, headers: dict, parameters: dict) -> None:
        """Assign elements to the created api object."""

        self.url = url
        self.headers = headers
        self.parameters = parameters


if __name__ == "__main__":
    print(f"Executing from {__name__}")
